local Gem = GemSocketsAddon;
local sockets = Gem.sockets;
local hovered = false;
local hover = false;
local good = false;

local infos = 
    {
        [1] = { ["name"] = "CharacterHeadSlot"},
        [2] = { ["name"] = "CharacterNeckSlot"},
        [3] = { ["name"] = "CharacterShoulderSlot"},
        [4] = { ["name"] = "CharacterShirtSlot"},
        [5] = { ["name"] = "CharacterChestSlot"},
        [6] = { ["name"] = "CharacterWaistSlot"},
        [7] = { ["name"] = "CharacterLegsSlot"},
        [8] = { ["name"] = "CharacterFeetSlot"},
        [9] = { ["name"] = "CharacterWristSlot"},
        [10] = { ["name"] = "CharacterHandsSlot"},
        [11] = { ["name"] = "CharacterFinger0Slot"},
        [12] = { ["name"] = "CharacterFinger1Slot"},
        [13] = { ["name"] = "CharacterTrinket0Slot"},
        [14] = { ["name"] = "CharacterTrinket1Slot"},
        [15] = { ["name"] = "CharacterBackSlot"},
        [16] = { ["name"] = "CharacterMainHandSlot"},
        [17] = { ["name"] = "CharacterSecondaryHandSlot"},
        [19] = { ["name"] = "CharacterTabardSlot"}
    };

local gemID = {};
local gemID_old = {};
local origsEnter = {};
local origsLeave = {};
local ft = {};

function sockets.showGameTooltip(self, motion)
    hovered = self.name;
    if self.link then
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
        GameTooltip:SetHyperlink(self.link);
        GameTooltip:Show();
    end
end

function sockets.hideGameTooltip(self, motion)
    GameTooltip:Hide();
end

function sockets.setTooltip()
    for id, item in pairs(infos) do
        local continue = true;
        frame = _G[item.name];
        itemLink = GetInventoryItemLink("player", id);

        if itemLink == nil then
            continue = true;
        end
        if continue then
            good = true;
            frame.id = id;
            frame.name = item.name;
            origsEnter[item.name] = frame:GetScript("OnEnter");
            origsLeave[item.name] = frame:GetScript("OnLeave");
            frame:SetScript("OnEnter", sockets.showGemsEnter);
            frame:SetScript("OnUpdate", sockets.showGemsUpdate);
            frame:SetScript("OnLeave", sockets.showGemsLeave);   
        end
    end
end

function sockets.showGemsEnter(self, motion)
    hovered = self.name;
    hover = true;
    if origsEnter[self.name] then return origsEnter[self.name](self, motion) end
end

function sockets.showGemsLeave(self, motion)
    hovered = false;
    if origsLeave[self.name] then return origsLeave[self.name](self, motion) end
end


function sockets.showGemsUpdate(self, motion)
    self.link = GetInventoryItemLink("player", self.id);
    if ( IsShiftKeyDown() and hovered == self.name and self.link ~= nil) then 
        if hover then
            GameTooltip:Hide();
            hover = false;
        end
        loadGems();

        i = 0;
        local canGem = false;
        local slotID = self.id;
        local itemLink = GetInventoryItemLink("player", slotID);
        
        local stats = GetItemStats(itemLink);
        if stats ~= nil then
            for key, val in pairs(stats) do
                if (string.find(key, "EMPTY_SOCKET_")) then
                    canGem = true;
                end
            end
        end
        
        if canGem then       
            local lastFrame = self;
            local i = 0;
            local m = 0;
            for id, item in pairs(gemID) do
                local tmpPos = 0;
                gamname = item.name;
                
                local gem = UseContainerItemByName(gamname, 0);
                
                if gem ~= false then
                    i = i + 1;
                    if i == 5 then
                        m = m + 1;
                        tmpPos = 0 - (lastFrame:GetHeight() * m);
                        lastFrame = self;
                        i = 1;
                    end
                    if ft[id] == nil then
                        ft[id] = CreateFrame("Button", nil, lastFrame);
                    end
                    ft[id]:SetWidth(50);
                    ft[id]:SetHeight(lastFrame:GetHeight());
                    
                    local t = ft[id]:CreateTexture(nil,"OVERLAY");
                    t:SetTexture(item.icon);
                    t:SetAllPoints(ft[id]);
                    ft[id].texture = t;
                    
                    ft[id]:SetPoint("BOTTOMLEFT", lastFrame, "BOTTOMLEFT", lastFrame:GetWidth(), tmpPos);
                    lastFrame = ft[id];
                    
                    ft[id].name = self.name;
                    ft[id].link = item.link;
                    ft[id]:SetScript("OnEnter", sockets.showGameTooltip);
                    ft[id]:SetScript("OnLeave", sockets.hideGameTooltip);
                    ft[id]:SetScript("OnClick", function (self, button, down)
                        gamname = item.name;
                        bag = UseContainerItemByName(gamname, 0);
                        slot = UseContainerItemByName(gamname, 1);
                        if bag ~= false and slot ~= false then
                            SocketInventoryItem(slotID);
                            PickupContainerItem(bag, slot);
                            ClickSocketButton(1);
                            AcceptSockets();
                            CloseSocketInfo();
                        end    
                    end);
                    ft[id]:Show();
                end
            end
        end
    end
    if not IsShiftKeyDown() then
        pcall(hideFrame);
    end
end

function hideFrame()
    for id, item in pairs(ft) do
        item:Hide();
    end
end

function loadGems()
    gemID = {};
    for bag = 0,4 do
        for slot = 1,GetContainerNumSlots(bag) do
            local itemL = GetContainerItemLink(bag,slot);
            if itemL then
                local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemIcon, itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(itemL);
                if itemClassID == 3 then
                    local arrayItem = { ["link"] = itemL, ["name"] = itemName, ["icon"] = itemIcon };
                    gemID[itemIcon] = arrayItem;        
                end
            end
        end
    end
end

function UseContainerItemByName(search, xy)
    for bag = 0,4 do
      for slot = 1,GetContainerNumSlots(bag) do
        local item = GetContainerItemLink(bag,slot);
        if item and item:find(search) then
            if xy == 0 then
                return bag;
            end
            if xy == 1 then
                return slot;
            end
        end
      end
    end
    return false;
  end

function sockets:init()
    sockets.setTooltip();
    if good == false then
        C_Timer.After(5, function()
            sockets.setTooltip();
        end)
    end
    if good == false then
        C_Timer.After(30, function()
            sockets.setTooltip();
        end)
    end
    if good == false then
        C_Timer.After(60, function()
            sockets.setTooltip();
        end)
    end
end